//
//  Path.m
//  Demine
//
//  Created by Michael Heyeck on 3/10/10.
//  Copyright 2010 Fair Oaks Labs, Inc.. All rights reserved.
//

#import "Path.h"


static void saveApplier(void* info, const CGPathElement* element)
{
	NSMutableArray* a = (NSMutableArray*) info;
	
	int nPoints;
	switch (element->type)
	{
		case kCGPathElementMoveToPoint:
			nPoints = 1;
			break;
		case kCGPathElementAddLineToPoint:
			nPoints = 1;
			break;
		case kCGPathElementAddQuadCurveToPoint:
			nPoints = 2;
			break;
		case kCGPathElementAddCurveToPoint:
			nPoints = 3;
			break;
		case kCGPathElementCloseSubpath:
			nPoints = 0;
			break;
		default:
			[a replaceObjectAtIndex:0 withObject:[NSNumber numberWithBool:NO]];
			return;
	}
	
	NSNumber* type = [NSNumber numberWithInt:element->type];
	NSMutableArray* points = [NSMutableArray arrayWithCapacity:nPoints];
	for (int i = 0; i < nPoints; i++)
	{
		NSNumber* nx = [NSNumber numberWithFloat:element->points[i].x];
		NSNumber* ny = [NSNumber numberWithFloat:element->points[i].y];
		[points addObject:[NSDictionary dictionaryWithObjectsAndKeys:nx,@"x",ny,@"y",nil]];
	}
	//NSData* points = [NSData dataWithBytes:element->points length:nPoints*sizeof(CGPoint)];
	
	[a addObject:[NSDictionary dictionaryWithObjectsAndKeys:type,@"type",points,@"points",nil]];
}


@implementation Path

@synthesize drawMode;
@synthesize strokeWidth;
@synthesize color;

- (CGPathRef)path
{
	return path;
}

- (void)setPath:(CGPathRef)newPath
{
	if (path != newPath)
	{
		CGPathRelease(path);
		path = CGPathRetain(newPath);
	}
}


- (id)init
{
    if (self = [super init])
	{
		self.drawMode		= kCGPathStroke;
		self.strokeWidth	= 10;
		self.color			= [UIColor blackColor];
    }
    return self;
}


- (id)copyWithZone:(NSZone*)zone
{
	Path* copy = [super copyWithZone:zone];

	copy->drawMode		= drawMode;
	copy->strokeWidth	= strokeWidth;
	copy->color			= [color retain];
	copy->path			= CGPathRetain(path);

	return copy;
}


- (BOOL)saveToFile:(NSString*)fileName
{
	if (!path) return NO;
	
	// Convert path to an array
	NSMutableArray* a = [NSMutableArray arrayWithObject:[NSNumber numberWithBool:YES]];
	CGPathApply(path, a, saveApplier);
	if (![[a objectAtIndex:0] boolValue])
	{
		return NO;
	}

	// Write path
	return [a writeToFile:fileName atomically:YES];
}


- (BOOL)loadFromFile:(NSString*)fileName
{
	// Read path
	NSArray* a = [NSArray arrayWithContentsOfFile:fileName];
	if (!a) return NO;

	// Recreate (and store) path
	CGPoint points[3];
	CGMutablePathRef p = CGPathCreateMutable();
	for (NSInteger i = 1, l = [a count]; i < l; i++)
	{
		NSDictionary* d = [a objectAtIndex:i];

		int nPoints = 0;
		for (NSDictionary* pt in [d objectForKey:@"points"])
		{
			points[nPoints].x = [[pt objectForKey:@"x"] floatValue];
			points[nPoints].y = [[pt objectForKey:@"y"] floatValue];
			nPoints++;
		}
		//CGPoint* points = (CGPoint*) [[d objectForKey:@"points"] bytes];
		
		switch ([[d objectForKey:@"type"] intValue])
		{
			case kCGPathElementMoveToPoint:
				CGPathMoveToPoint(p, NULL, points[0].x, points[0].y);
				break;
			case kCGPathElementAddLineToPoint:
				CGPathAddLineToPoint(p, NULL, points[0].x, points[0].y);
				break;
			case kCGPathElementAddQuadCurveToPoint:
				CGPathAddQuadCurveToPoint(p, NULL, points[0].x, points[0].y, points[1].x, points[1].y);
				break;
			case kCGPathElementAddCurveToPoint:
				CGPathAddCurveToPoint(p, NULL, points[0].x, points[0].y, points[1].x, points[1].y, points[2].x, points[2].y);
				break;
			case kCGPathElementCloseSubpath:
				CGPathCloseSubpath(p);
				break;
			default:
				CGPathRelease(p);
				return NO;
		}
	}
	self.path = p;
	CGPathRelease(p);

	// Signal success
	return YES;
}


- (void)renderToContext:(CGContextRef)context
{
	if (!path) return;
	
	CGContextAddPath(context, path);
	[color set];
	CGContextSetLineWidth(context, strokeWidth);
	CGContextDrawPath(context, drawMode);
}


- (void)dealloc
{
	[color release];
	CGPathRelease(path);
	[super dealloc];
}

@end
