//
//  Puzzle.h
//  Demine
//
//  Created by Michael Heyeck on 4/9/10.
//  Copyright 2010 Fair Oaks Labs, Inc.. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "ManagedPuzzle.h"


enum _PuzzleStatus
{
	PuzzleStatusUnsolved	= 0,
	PuzzleStatusSolved,
	PuzzleStatusFailed,
};
typedef enum _PuzzleStatus PuzzleStatus;


@interface Puzzle : NSObject
{
	NSUInteger		w;
	NSUInteger		h;
	NSUInteger		nMines;
	NSUInteger		nUncovered;
	NSUInteger		nFlagged;
	NSTimeInterval	elapsedTime;
	PuzzleStatus	status;
	
	char*			cells;
}

@property (nonatomic, assign, readonly) NSUInteger w;
@property (nonatomic, assign, readonly) NSUInteger h;
@property (nonatomic, assign, readonly) NSUInteger nMines;
@property (nonatomic, assign, readonly) NSUInteger nUncovered;
@property (nonatomic, assign, readonly) NSUInteger nFlagged;
@property (nonatomic, assign, readonly) NSTimeInterval elapsedTime;
@property (nonatomic, assign, readonly) PuzzleStatus status;

- (id)initWithWidth:(NSUInteger)w height:(NSUInteger)h mines:(NSUInteger)m;
- (id)initWithManagedPuzzle:(ManagedPuzzle*)puzzle;
- (void)saveToManagedPuzzle:(ManagedPuzzle*)puzzle;

- (void)generate;
- (NSData*)revealCell:(CGPoint)pos withCellBox:(CGRect)cellBox;
- (NSData*)toggleFlagOnCell:(CGPoint)pos withCellBox:(CGRect)cellBox;
- (void)addPlaytime:(NSTimeInterval)dt;

- (BOOL)isLegalCell:(CGPoint)pos;
- (BOOL)isRevealedCell:(CGPoint)pos;
- (BOOL)isFlaggedCell:(CGPoint)pos;
- (BOOL)isMinedCell:(CGPoint)pos;
- (NSUInteger)neighborsOfCell:(CGPoint)pos;

@end
