//
//  Store.m
//  Demine
//
//  Created by Michael Heyeck on 4/30/10.
//  Copyright 2010 Fair Oaks Labs, Inc.. All rights reserved.
//

#import "Item.h"
#import "Store.h"


@interface Store ()

@property (nonatomic, retain) Reachability* reachability;
@property (nonatomic, retain) SKRequest* request;
@property (nonatomic, retain) NSArray* products;
@property (nonatomic, retain, readonly) NSNumberFormatter* priceFormatter;

- (void)reachabilityChanged:(NSNotification*)note;
- (void)restore;
- (void)dismiss;

@end


@implementation Store

@synthesize reachability;
@synthesize request;
@synthesize products;
@synthesize tableView;
@synthesize loadingView;
@synthesize loadingLabel;

- (NSNumberFormatter*)priceFormatter
{
	if (!priceFormatter)
	{
		priceFormatter = [[NSNumberFormatter alloc] init];
		priceFormatter.formatterBehavior = NSNumberFormatterBehavior10_4;
		priceFormatter.numberStyle = NSNumberFormatterCurrencyStyle;
	}
	return priceFormatter;
}


/*
- (id)initWithStyle:(UITableViewStyle)style {
    // Override initWithStyle: if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
    if (self = [super initWithStyle:style]) {
    }
    return self;
}
*/


- (void)viewDidLoad
{
    [super viewDidLoad];

	// Nav. bar setup
	self.navigationItem.leftBarButtonItem = [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemCancel target:self action:@selector(dismiss)] autorelease];
	self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc] initWithTitle:@"Restore" style:UIBarButtonItemStyleBordered target:self action:@selector(restore)] autorelease];
	self.navigationItem.rightBarButtonItem.enabled = NO;
	self.title = @"Store";

	// Check network
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(reachabilityChanged:)
												 name:kReachabilityChangedNotification
											   object:nil];
	self.reachability = [Reachability reachabilityWithHostName:@"www.apple.com"];
	[self.reachability startNotifer];

	// Populate store
	NSArray* a = [NSArray arrayWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"Products" ofType:@"plist"]];
	self.request = [[[SKProductsRequest alloc] initWithProductIdentifiers:[NSSet setWithArray:a]] autorelease];
	self.request.delegate = self;
	[self.request start];

	// Observe restore operations
	[[SKPaymentQueue defaultQueue] addTransactionObserver:self];
}


- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
	
	[self.tableView deselectRowAtIndexPath:[self.tableView indexPathForSelectedRow] animated:NO];
}
/*
- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
}
*/
/*
- (void)viewWillDisappear:(BOOL)animated {
	[super viewWillDisappear:animated];
}
*/
/*
- (void)viewDidDisappear:(BOOL)animated {
	[super viewDidDisappear:animated];
}
*/

/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[[SKPaymentQueue defaultQueue] removeTransactionObserver:self];
	
	[reachability release];
	[request cancel];
	[request release];
	[products release];
	[priceFormatter release];
	[tableView release];
	[loadingView release];
	[loadingLabel release];
    [super dealloc];
}

#pragma mark Table view data source methods

- (NSInteger)numberOfSectionsInTableView:(UITableView*)tableView
{
    return 1;
}


// Customize the number of rows in the table view.
- (NSInteger)tableView:(UITableView*)tableView numberOfRowsInSection:(NSInteger)section
{
    return [products count];
}


// Customize the appearance of table view cells.
- (UITableViewCell*)tableView:(UITableView*)aTableView cellForRowAtIndexPath:(NSIndexPath*)indexPath
{
    static NSString* CellIdentifier = @"Cell";
    
    UITableViewCell* cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil)
	{
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:CellIdentifier] autorelease];
		cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    }
	SKProduct* product = [products objectAtIndex:indexPath.row];
	
	NSNumberFormatter* pf = self.priceFormatter;
	pf.locale = product.priceLocale;
	
    // Set up the cell...
	cell.textLabel.text = product.localizedTitle;
	cell.detailTextLabel.text = [pf stringFromNumber:product.price];
	
    return cell;
}


/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/


/*
// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:YES];
    }   
    else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
    }   
}
*/


/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath {
}
*/


/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/

#pragma mark Table view delegate methods

- (void)tableView:(UITableView*)tableView didSelectRowAtIndexPath:(NSIndexPath*)indexPath
{
	Item* item = [[Item alloc] initWithNibName:@"Item" bundle:nil product:[products objectAtIndex:indexPath.row]];
	[self.navigationController pushViewController:item animated:YES];
	[item release];
}

#pragma mark Products request delegate methods

- (void)productsRequest:(SKProductsRequest*)aRequest didReceiveResponse:(SKProductsResponse*)response
{
	self.request = nil;
	self.products = response.products;

	self.navigationItem.rightBarButtonItem.enabled = YES;

	[self.tableView reloadData];
	self.loadingView.hidden = YES;
}

#pragma mark Store kit observer methods

- (void)paymentQueue:(SKPaymentQueue*)queue updatedTransactions:(NSArray*)transactions
{
	// Not relevant to this class; only implemented because it's mandatory
}


- (void)paymentQueueRestoreCompletedTransactionsFinished:(SKPaymentQueue*)queue
{
	if (self.products)
	{
		self.navigationItem.rightBarButtonItem.enabled = YES;
		self.loadingView.hidden = YES;
	}
}


- (void)paymentQueue:(SKPaymentQueue*)queue restoreCompletedTransactionsFailedWithError:(NSError*)error
{
	if (self.products)
	{
		self.navigationItem.rightBarButtonItem.enabled = YES;
		self.loadingView.hidden = YES;
	}
}

#pragma mark Alert view delegate methods

- (void)alertView:(UIAlertView*)av clickedButtonAtIndex:(NSInteger)buttonIndex
{
	[self dismiss];
}

#pragma mark Extension methods

- (void)reachabilityChanged:(NSNotification*)note
{
    Reachability* r = [note object];
	NetworkStatus ns = r.currentReachabilityStatus;
	
	if (ns == NotReachable)
	{
		NSString* msg = @"Network problems have rendered the iTunes store inaccessible; please try again later.";
		UIAlertView* av = [[UIAlertView alloc] initWithTitle:nil
													 message:msg
													delegate:self
										   cancelButtonTitle:@"Ok"
										   otherButtonTitles:nil];
		[av show];
		[av release];
	}
}


- (void)restore
{
	self.loadingLabel.text = @"Restoring";

	self.navigationItem.rightBarButtonItem.enabled = NO;
	self.loadingView.hidden = NO;
	
	[[SKPaymentQueue defaultQueue] restoreCompletedTransactions];
}


- (void)dismiss
{
	[self.navigationController.parentViewController dismissModalViewControllerAnimated:YES];
}

@end
